# -*- coding: utf-8 -*-
"""
Módulo que contiene una clase para leer una ruta GPS
de un fichero.

@author: Michael
@date:   Abr 2019
"""

import math
from typing import List
from typing import Tuple
import xml.etree.ElementTree as ET

class Track:
    """
    Clase que contiene una ruta GPS obtenida de un fichero y
    operaciones para obtener latitudes, longitudes y elevaciones,
    asi como hacer calculos de distancia con puntos

    Attributes
        __puntos: lista de tuplas con la latitud, longitud y elevación
                  de cada punto de la ruta
    """

    def __init__(self, nombre_fichero: str):
        """
        Carga los datos de un fichero GPX, cuyo nombre se pasa como argumento

        Attributes:
            nombre_fichero: el nombre del fichero que guarda la ruta GPX
        """
        # Crea el atributo vacío
        self.__puntos: List[Tuple[float, float, float]] = []
        # Lee el fichero
        tree = ET.parse(nombre_fichero)
        root = tree.getroot()
        n_s: str = '{http://www.topografix.com/GPX/1/1}'
        # Recorremos todos los tracks
        for track in root.findall(n_s+'trk'):
            # Recorremos todos los segmentos de track
            for seg in track.findall(n_s+'trkseg'):
                # Recorremos todos los puntos del segmento
                for point in seg.findall(n_s+'trkpt'):
                    lat: float = float(point.attrib['lat'])
                    lon: float = float(point.attrib['lon'])
                    ele: float = float(point.find(n_s+'ele').text)
                    self.__puntos.append((lat, lon, ele))

    @staticmethod
    def distancia(lat1: float, long1: float,
                  lat2: float, long2: float)-> float:
        """
        Retorna la distancia, en metros, desde el punto 1 al punto 2, dadas
        las latitudes y longitudes que se pasan como argumentos en grados

        Attributes:
            lat1   latitud del punto 1 en grados
            long1  longitud del punto 1 en grados
            lat2   latitud del punto 2 en grados
            long2  longitud del punto 2 en grados
        Returns:
            distancia entre el punto 1 y el punto 2 en metros
        """
        radio: float = 6_371_000.0 # metros
        lat1 = math.radians(lat1)
        long1 = math.radians(long1)
        lat2 = math.radians(lat2)
        long2 = math.radians(long2)
        sin_lat2: float = math.sin((lat2-lat1)/2)**2
        sin_long2: float = math.sin((long2-long1)/2)**2
        return 2*radio*math.asin(math.sqrt(sin_lat2+math.cos(lat1)*
                                           math.cos(lat2)*sin_long2))


    def get_latitudes(self)-> List[float]:
        """
        Retorna la lista de latitudes de los puntos del track
        """
        return [pto[0] for pto in self.__puntos]


    def get_longitudes(self)-> List[float]:
        """
        Retorna la lista de longitudes de los puntos del track
        """
        return [pto[1] for pto in self.__puntos]


    def get_elevaciones(self)-> List[float]:
        """
        Retorna la lista de elevaciones de los puntos del track
        """
        return [pto[2] for pto in self.__puntos]
